import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;


import org.junit.Before;
import org.junit.Test;

import at.jku.koppensteiner.androgios.nagios.NagiosInstance;
import at.jku.koppensteiner.androgios.nagios.model.Host;
import at.jku.koppensteiner.androgios.nagios.model.HostComment;
import at.jku.koppensteiner.androgios.nagios.model.HostDowntime;
import at.jku.koppensteiner.androgios.nagios.model.MonitoringModel;
import at.jku.koppensteiner.androgios.nagios.model.Outage;
import at.jku.koppensteiner.androgios.nagios.model.Service;
import at.jku.koppensteiner.androgios.nagios.model.ServiceDowntime;
import at.jku.koppensteiner.androgios.nagios.model.TacticalOverview;
import at.jku.koppensteiner.androgios.nagios.model.Process;
import at.jku.koppensteiner.androgios.parser.*;




public class ParserTest {

	private NagiosInstance instance;
	
	@Before
	public void prepare(){
		instance = new NagiosInstance(null, null, null);
	}
	

	
	@Test
	public void tac() throws IOException, ParserException{
		
		String html = getHtml("html/tac.html");
		TACParser parser = new TACParser(html, instance);
		parser.parse();
		
		TacticalOverview tac = instance.tacticalOverview;
		
		assertTrue(tac.updateSuccessful);
		
		assertFalse(tac.hasImportantHostProblem);
		assertFalse(tac.hasImportantOutageProblem);
		assertFalse(tac.hasImportantServiceProblem);
		assertFalse(tac.hasImportantProblem());
		
		assertFalse(tac.hasUnimportantHostProblem);
		assertFalse(tac.hasUnimportantOutageProblem);
		assertFalse(tac.hasUnimportantServiceProblem);
		
		assertEquals(tac.hostHealth, "100.0% Health");
		assertEquals(tac.serviceHealth, "100.0% Health");
		
		assertEquals(tac.outagesSummary.outages, "0 Outages");
		
		assertTrue(tac.hostStatusSummary.hostsDown.startsWith("0 Down"));
		assertTrue(tac.hostStatusSummary.hostsUp.startsWith("3 Up"));
		assertTrue(tac.hostStatusSummary.hostsPending.startsWith("0 Pending"));
		assertTrue(tac.hostStatusSummary.hostsUnreachable.startsWith("0 Unreachable"));
		
		assertTrue(tac.serviceStatusSummary.servicesCritical.startsWith("0 Critical"));
		assertTrue(tac.serviceStatusSummary.servicesOk.startsWith("7 Ok"));
		assertTrue(tac.serviceStatusSummary.servicesPending.startsWith("0 Pending"));
		assertTrue(tac.serviceStatusSummary.servicesUnknown.startsWith("0 Unknown"));
		assertTrue(tac.serviceStatusSummary.servicesWarning.startsWith("0 Warning"));
		
		assertTrue(tac.monitoringFeaturesSummary.hostsActiveChecks.startsWith("All Hosts Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.hostsEventHandlers.startsWith("All Hosts Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.hostsFlapDetection.startsWith("All Hosts Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.hostsFlapping.startsWith("No Hosts Flapping"));
		assertTrue(tac.monitoringFeaturesSummary.hostsNotifications.startsWith("All Hosts Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.hostsPassiveChecks.startsWith("All Hosts Enabled"));
		
		assertTrue(tac.monitoringFeaturesSummary.servicesActiveChecks.startsWith("All Services Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.servicesEventHandlers.startsWith("All Services Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.servicesFlapDetection.startsWith("All Services Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.servicesFlapping.startsWith("No Services Flapping"));
		assertTrue(tac.monitoringFeaturesSummary.servicesNotifications.startsWith("All Services Enabled"));
		assertTrue(tac.monitoringFeaturesSummary.servicesPassiveChecks.startsWith("All Services Enabled"));
		
	}
	
	
	@Test
	public void comments() throws IOException, ParserException{
		String html = getHtml("html/comments.html");
		Parser parser = new CommentsParser(html, instance);
		parser.parse();
		
		MonitoringModel model = instance.model;
		
		assertTrue(model.hostComments.size()==1);
		HostComment c = model.hostComments.get(0);
		assertTrue(c.author.equals("nagiosadmin"));
		assertTrue(c.comment.equals("test for parser"));
		assertTrue(c.entryTime.equals("18-02-2012 18:55:24"));
		assertTrue(c.hostName.equals("printer"));
		assertTrue(c.expires.equals("N/A"));
		
	}
	
	@Test
	public void downtimes() throws IOException, ParserException{
		String html = getHtml("html/downtimes.html");
		Parser parser = new DowntimesParser(html, instance);
		parser.parse();
		
		MonitoringModel model = instance.model;
		
		assertTrue(model.hostDowntimes.size()==1);
		assertTrue(model.serviceDowntimes.size()==1);
		
		HostDowntime h = model.hostDowntimes.get(0);
		ServiceDowntime s = model.serviceDowntimes.get(0);
		
		assertTrue(h.hostName.equals("printer"));
		assertTrue(h.entryTime.equals("23-05-2012 14:58:12"));
		assertTrue(h.startTime.equals("23-05-2012 14:58:01"));
		assertTrue(h.endTime.equals("23-05-2012 16:58:01"));
		assertTrue(h.triggerId.equals("N/A"));
		
		assertTrue(s.hostName.equals("printer"));
		assertTrue(s.serviceName.equals("PING"));
		assertTrue(s.entryTime.equals("23-05-2012 15:09:17"));
		assertTrue(s.startTime.equals("23-05-2012 15:09:10"));
		assertTrue(s.endTime.equals("23-05-2012 17:09:10"));
		assertTrue(s.triggerId.equals("N/A"));
	}
	
	@Test
	public void hostDetail() throws IOException, ParserException{
		
		Host host = new Host();
		host.hostName = "printer";
		instance.model.hosts.add(host);
		String html = getHtml("html/hostDetail.html");
		Parser parser = new HostDetailParser(html, instance, "printer");
		parser.parse();
		
		host = instance.model.hosts.get(0);
		
		assertTrue(host.hostStatusDetail.contains("UP"));
		assertTrue(host.hostStatusDetail.contains("(for 22d 1h 4m 53s)"));
		assertTrue(host.statusInformation.equals("PING OK - Packet loss = 0%, RTA = 0.79 ms"));
		assertTrue(host.performanceData.equals("rta=0.786000ms;5000.000000;5000.000000;0.000000 pl=0%;100;100;0"));
		assertTrue(host.currentAttempt.equals("1/10  (HARD state)"));
		assertTrue(host.lastCheckTime.equals("23-05-2012 14:55:09"));
		assertTrue(host.checkType.equals("ACTIVE"));
		assertTrue(host.checkLatencyDuration.equals("0.070 / 0.019 seconds"));
		
		assertTrue(host.nextScheduledActiveCheck.equals("23-05-2012 15:00:19"));
		assertTrue(host.lastStateChange.equals("01-05-2012 13:51:09"));
		assertTrue(host.lastNotification.equals("01-05-2012 13:51:09 (notification 0)"));
		assertTrue(host.isThisHostFlapping.equals("  NO   (0.00% state change)"));
		assertTrue(host.inScheduledDowntime.equals("  NO  "));
		assertTrue(host.lastUpdate.equals("23-05-2012 14:55:59  ( 0d 0h 0m 3s ago)"));
		
		assertTrue(host.activeChecks.equals("  ENABLED  "));
		assertTrue(host.passiveChecks.equals("  ENABLED  "));
		assertTrue(host.obsessing.equals("  ENABLED  "));
		assertTrue(host.notifications.equals("  ENABLED  "));
		assertTrue(host.eventHandler.equals("  ENABLED  "));
		assertTrue(host.flapDetection.equals("  ENABLED  "));
		
		
	}
	
	@Test
	public void hostGroups() throws IOException, ParserException{
		String html = getHtml("html/hostGroups.html");
		Parser parser = new HostGroupsParser(html, instance);
		parser.parse();
		
		assertTrue(instance.model.hostGroups.size()==5);
		
		assertTrue(instance.model.hostGroups.get(0).hostStatusSummary.hostsUp.equals("3 UP"));
		assertTrue(instance.model.hostGroups.get(0).serviceStatusSummary.servicesOk.equals("9 OK"));
		
		assertTrue(instance.model.hostGroups.get(1).hostStatusSummary.hostsUp.equals("1 UP"));
		assertTrue(instance.model.hostGroups.get(1).serviceStatusSummary.servicesOk.equals("7 OK"));
		
		assertTrue(instance.model.hostGroups.get(2).hostStatusSummary.hostsUp.equals("1 UP"));
		assertTrue(instance.model.hostGroups.get(2).serviceStatusSummary.servicesOk.equals("7 OK"));
		
		assertTrue(instance.model.hostGroups.get(3).hostStatusSummary.hostsUp.equals("3 UP"));
		assertTrue(instance.model.hostGroups.get(3).serviceStatusSummary.servicesOk.equals("9 OK"));
		
		assertTrue(instance.model.hostGroups.get(4).hostStatusSummary.hostsUp.equals("1 UP"));
		assertTrue(instance.model.hostGroups.get(4).serviceStatusSummary.servicesOk.equals("7 OK"));
		
		
	}
	
	@Test
	public void hostList() throws IOException, ParserException{
		String html = getHtml("html/hostlist.html");
		Parser parser = new HostListParser(html, instance, false, false);
		parser.parse();
		
		assertTrue(instance.model.hosts.size()==3);
		
		assertTrue(instance.model.hosts.get(0).hostName.equals("gateway"));
		assertTrue(instance.model.hosts.get(0).hostStatus.equals("UP"));
		
		assertTrue(instance.model.hosts.get(1).hostName.equals("localhost"));
		assertTrue(instance.model.hosts.get(1).hostStatus.equals("UP"));
		
		assertTrue(instance.model.hosts.get(2).hostName.equals("printer"));
		assertTrue(instance.model.hosts.get(2).hostStatus.equals("UP"));
		
	}
	
	@Test
	public void outages() throws IOException, ParserException{
		String html = getHtml("html/outages.html");
		Parser parser = new OutagesParser(html, instance);
		parser.parse();
		
		assertTrue(instance.model.outages.size()==2);
		Outage o = instance.model.outages.get(0);
		
		assertTrue(o.host.equals("testRouter1"));
		assertTrue(o.nrHostsAffected.equals("3"));
		assertTrue(o.nrServicesAffected.equals("3"));
		assertTrue(o.state.equals("DOWN"));
		assertTrue(o.stateDuration.equals("0d 0h 0m 43s"));
		assertTrue(o.severity.equals("3"));
		
	}
	
	@Test
	public void processDetail() throws IOException, ParserException{
		String html = getHtml("html/process.html");
		Parser parser = new ProcessDetailParser(html, instance);
		parser.parse();
		
		Process p = instance.model.process;
		
		assertTrue(p.programVersion.equals("3.2.0"));
		assertTrue(p.programStartTime.equals("11-05-2012 12:27:19"));
		assertTrue(p.totalRunningTime.equals("12d 2h 34m 3s"));
		assertTrue(p.lastExternalCommandCheck.equals("23-05-2012 15:01:19"));
		assertTrue(p.lastLogFileRotation.equals("23-05-2012 00:00:00"));
		assertTrue(p.nagiosPID.equals("1155"));
		assertTrue(p.notificationsEnabled.equals("  YES  "));
		assertTrue(p.serviceChecksBeingExecuted.equals("  YES  "));
		assertTrue(p.passiveServiceChecksBeingAccepted.equals("  YES  "));
		assertTrue(p.hostChecksBeingExecuted.equals("  YES  "));
		assertTrue(p.passiveHostChecksBeingAccepted.equals("  YES  "));
		assertTrue(p.eventHandlersEnabled.equals("Yes"));
		assertTrue(p.obsessingOverServices.equals("No"));
		assertTrue(p.obsessingOverHosts.equals("No"));
		assertTrue(p.flapDetectionEnabled.equals("Yes"));
		assertTrue(p.performanceDataBeingProcessed.equals("No"));
		
	}
	
	@Test
	public void serviceDetail() throws IOException, ParserException{
		Host host = new Host();
		Service service = new Service();
		service.serviceName = "PING";
		host.hostName = "printer";
		host.hostServices.add(service);
		instance.model.hosts.add(host);
		String html = getHtml("html/serviceDetail.html");
		Parser parser = new ServiceDetailParser(html, instance, "printer", "PING");
		parser.parse();
		
		Service s = instance.model.hosts.get(0).hostServices.get(0);
		
		assertTrue(s.currentStatusDetail.equals("  OK   (for 22d 1h 16m 27s)"));
		assertTrue(s.statusInformation.equals("PING OK - Packet loss = 0%, RTA = 0.74 ms"));
		assertTrue(s.performanceData.equals("rta=0.741000ms;100.000000;500.000000;0.000000 pl=0%;20;60;0"));
		assertTrue(s.currentAttempt.equals("1/4  (HARD state)"));
		assertTrue(s.lastCheckTime.equals("23-05-2012 15:05:48"));
		assertTrue(s.checkType.equals("ACTIVE"));
		assertTrue(s.checkLatencyDuration.equals("0.120 / 4.018 seconds"));
		assertTrue(s.nextScheduledCheck.equals("23-05-2012 15:10:48"));
		assertTrue(s.lastStateChange.equals("01-05-2012 13:50:48"));
		assertTrue(s.lastNotification.equals("N/A (notification 0)"));
		assertTrue(s.isThisServiceFlapping.equals("  NO   (0.00% state change)"));
		assertTrue(s.inScheduledDowntime.equals("  NO  "));
		assertTrue(s.lastUpdate.equals("23-05-2012 15:07:09  ( 0d 0h 0m 6s ago)"));
		
		
		assertTrue(s.activeChecks.equals("  ENABLED  "));
		assertTrue(s.passiveChecks.equals("  ENABLED  "));
		assertTrue(s.obsessing.equals("  ENABLED  "));
		assertTrue(s.notifications.equals("  ENABLED  "));
		assertTrue(s.eventHandler.equals("  ENABLED  "));
		assertTrue(s.flapDetection.equals("  ENABLED  "));
		
	}
	
	@Test
	public void serviceGroups() throws IOException, ParserException{
		String html = getHtml("html/serviceGroups.html");
		Parser parser = new ServiceGroupsParser(html, instance);
		parser.parse();
		
		assertTrue(instance.model.serviceGroups.size()==2);
		
		assertTrue(instance.model.serviceGroups.get(0).hostStatusSummary.hostsUp.equals("2 UP"));
		assertTrue(instance.model.serviceGroups.get(0).serviceStatusSummary.servicesOk.equals("3 OK"));
		
		assertTrue(instance.model.serviceGroups.get(1).hostStatusSummary.hostsUp.equals("1 UP"));
		assertTrue(instance.model.serviceGroups.get(1).serviceStatusSummary.servicesOk.equals("1 OK"));
		
	}
	
	@Test
	public void serviceList() throws IOException, ParserException{
		String html = getHtml("html/serviceList.html");
		Parser parser = new ServiceListParser(html, instance,false,false);
		parser.parse();
		
		
		assertTrue(instance.model.hosts.size()==3);
		
		assertTrue(instance.model.hosts.get(0).hostServices.size()==1);
		assertTrue(instance.model.hosts.get(1).hostServices.size()==7);
		assertTrue(instance.model.hosts.get(2).hostServices.size()==1);
		
		assertTrue(instance.model.hosts.get(0).hostName.equals("gateway"));
		assertTrue(instance.model.hosts.get(0).hostServices.get(0).serviceName.equals("PING"));
		
		assertTrue(instance.model.hosts.get(1).hostName.equals("localhost"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(0).serviceName.equals("Current Load"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(1).serviceName.equals("Current Users"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(2).serviceName.equals("Disk Space"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(3).serviceName.equals("HTTP"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(4).serviceName.equals("PING"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(5).serviceName.equals("SSH"));
		assertTrue(instance.model.hosts.get(1).hostServices.get(6).serviceName.equals("Total Processes"));
		
		assertTrue(instance.model.hosts.get(2).hostName.equals("printer"));
		assertTrue(instance.model.hosts.get(2).hostServices.get(0).serviceName.equals("PING"));
		
	}
	
	
	
	
	private String getHtml(String path) throws IOException{

		BufferedReader reader = new BufferedReader(new FileReader(path));
		
		String line = "";
		StringBuilder sb = new StringBuilder();
		
		while( ( line = reader.readLine())!=null ){
			sb.append(line);
		}
		return sb.toString();
	}
	
	
}
